#include <stdio.h>
#include "kernel/OS.h"
#include "support/Locker.h"

// Our two locks... having the threads lock them in differing orders causes a
// deadlock to (occasionally) occur!
BLocker lock1;
BLocker lock2;

long threadFunc(void * arg);
long threadFunc(void * arg)
{
   while(1)
   {
      printf("Thread %lu about to lock 1...\n", find_thread(NULL));
      lock1.Lock();

      printf("Thread %lu about to lock 2...\n", find_thread(NULL));
      lock2.Lock();

      // Doesn't matter which order you unlock in, as you never block while unlocking
      lock2.Unlock();
      lock1.Unlock();
      printf("Thread %lu unlocked both locks.\n");
   }
   return 0;
}

int main(int argc, char ** argv)
{
   printf("This app will demonstrate the problem of deadlocks.\n");
   printf("Eventually output will cease, as both threads are waiting for\n");
   printf("the other thread to unlock before they will unlock their own lock!\n");


   // Create the separate thread here...
   thread_id launchedThread = spawn_thread(threadFunc, "counter thread", B_NORMAL_PRIORITY, NULL); 
   if (launchedThread >= 0)
   {
      resume_thread(launchedThread);  // gotta do this to start it running

      while(1)
      {
         printf("Thread %lu about to lock 2...\n", find_thread(NULL));
         lock2.Lock();

         printf("Thread %lu about to lock 1...\n", find_thread(NULL));
         lock1.Lock();

         // Doesn't matter which order you unlock in, as you never block while unlocking
         lock2.Unlock();
         lock1.Unlock();
         printf("Thread %lu unlocked both locks.\n");
      }
   }
   else printf("Error launching thread!??!?\n");

   return 0; 
}
