/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <time.h>

#include "dataio/TCPSocketDataIO.h"
#include "iogateway/PortableMessageIOGateway.h"
#include "reflector/StorageReflectConstants.h"
#include "util/NetworkUtilityFunctions.h"

static void Kick(PortableMessageIOGateway & gw, const char * arg);
void Kick(PortableMessageIOGateway & gw, const char * arg)
{
   PortableMessage * msg = GetMessageFromPool(PR_COMMAND_KICK);
   PortableString str("/");
   str += arg;
   str += "/*";
   msg->AddString(PR_NAME_KEYS, str);
   gw.AddOutgoingMessage(PortableMessageRef(msg, GetMessagePool(), GetMessageRefPool()));
   printf("Kicking users matching pattern [%s]\n", str.Cstr());
}

static void Ban(PortableMessageIOGateway & gw, const char * arg, bool unBan);
void Ban(PortableMessageIOGateway & gw, const char * arg, bool unBan)
{
   PortableMessage * msg = GetMessageFromPool(unBan ? PR_COMMAND_REMOVEBANS : PR_COMMAND_ADDBANS);
   msg->AddString(PR_NAME_KEYS, arg);
   gw.AddOutgoingMessage(PortableMessageRef(msg, GetMessagePool(), GetMessageRefPool()));
   if (unBan) printf("Removing ban patterns that match pattern [%s]\n", arg);
         else printf("Adding ban pattern [%s]\n", arg);
}


// This is a little admin program, useful for kicking, banning, or unbanning users
// without having to restart the MUSCLE server.  Example command line:
//   admin server=muscleserver.mycompany.com kick=192.168.0.23 ban=16.25.29.2 kickban=1.2.3.4 unban=1.2.3.4 ban=2.3.4.5 ban=3.4.5.*
// Note that you can only do this if your IP address has the requisite privileges on
// the MUSCLE server!  (i.e. to ban, the server must have been run with an argument
// like privban=your.ip.address)
int main(int argc, char ** argv)
{
   char * hostName = "localhost";

   // First, find out if there is a server specified.
   for (int i=1; i<argc; i++)
   {
      char * next = argv[i];
      if (strncmp(next, "server=", 7) == 0) hostName = &next[7];
      else if (strcmp(next, "help") == 0)
      {
         printf("This program lets you send admin commands to a running MUSCLE server.\n");
         printf("Note that the MUSCLE server won't listen to your commands unless your ip address was\n");
         printf("specified as a privileged IP address in its command line arguments [e.g. ./muscled privall=your.IP.address]\n");
         printf("Usage:  admin [server=localhost] [ban=pattern] [unban=pattern] [kick=pattern] [kickban=pattern]\n");
         exit(0);
      }
   }
   
   char * colon = strchr(hostName, ':');
   int port = (colon) ? atoi(colon+1) : 2960;
   if (colon) *colon = '\0';

   int s = Connect(hostName, port, "admin");
   if (s < 0) 
   {
      printf("(run 'admin help' for arguments)\n");
      exit(10);
   }

   PortableMessageIOGateway gw(new TCPSocketDataIO(s, false));

   // Now generate all commands...
   for (int j=1; j<argc; j++)
   {
      char * line = argv[j];
      char * colon = strchr(line, '=');
      if (colon)
      {
         char * arg = colon+1;
         *colon = '\0';
         if (strcmp(line, "kick") == 0) Kick(gw, arg);
         else if (strcmp(line, "ban") == 0) Ban(gw, arg, false);
         else if (strcmp(line, "unban") == 0) Ban(gw, arg, true);
         else if (strcmp(line, "kickban") == 0)
         {
            Kick(gw, arg);
            Ban(gw, arg, false);
         }
      }
   }

   // and send them
   while(true)
   {
      if (gw.HasBytesToOutput() == false) 
      {
         printf("All done, exiting\n");
         break;
      }
      if (gw.DoOutput() != B_NO_ERROR)
      {
         printf("Error writing, exiting!\n");
         break;
      }
   }
   return 0;
}
